package com.onsiteservice.admin.service.service;

import com.github.pagehelper.PageHelper;
import com.google.common.collect.Lists;
import com.onsiteservice.admin.controller.service.dto.PageServiceValuatorDTO;
import com.onsiteservice.admin.controller.service.dto.SaveServiceValuatorDTO;
import com.onsiteservice.admin.controller.service.dto.UpdateServiceValuatorDTO;
import com.onsiteservice.admin.controller.service.vo.ServiceValuatorVO;
import com.onsiteservice.admin.mapper.service.ServiceValuatorBizMapper;
import com.onsiteservice.constant.enums.BizCodeEnum;
import com.onsiteservice.constant.enums.ServiceUserTypeEnum;
import com.onsiteservice.core.exception.ServiceException;
import com.onsiteservice.core.result.Result;
import com.onsiteservice.core.result.ResultGenerator;
import com.onsiteservice.dao.common.AbstractMapper;
import com.onsiteservice.dao.common.page.PageInfoVO;
import com.onsiteservice.dao.mapper.service.ServiceValuatorAssignMapper;
import com.onsiteservice.dao.mapper.user.UserMapper;
import com.onsiteservice.entity.service.ServiceValuator;
import com.onsiteservice.entity.service.ServiceValuatorAssign;
import com.onsiteservice.entity.service.ServiceWorker;
import com.onsiteservice.entity.user.User;
import com.onsiteservice.util.AttrCopyUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import tk.mybatis.mapper.entity.Condition;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author 潘维吉
 * @date 2022-07-11 16:07
 * @description ServiceValuatorService服务类
 */
@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class ServiceValuatorService extends AbstractMapper<ServiceValuator> {


    @Resource
    private ServiceValuatorBizMapper serviceValuatorBizMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private ServiceValuatorAssignMapper serviceValuatorAssignMapper;
;


    /**
     * 分页查询列表
     */
    public Result<PageInfoVO<ServiceValuatorVO>> getPage(PageServiceValuatorDTO dto) {
        PageHelper.startPage(dto.getPage(), dto.getSize());

        List<ServiceValuator> serviceValuatorList = serviceValuatorBizMapper.selectServiceValuatorPage(dto);
        List<ServiceValuatorVO> serviceValuatorVOList = serviceValuatorList.parallelStream().map(e -> AttrCopyUtils.copy(e, new ServiceValuatorVO())).collect(Collectors.toList());

        return ResultGenerator.success(new PageInfoVO<>(serviceValuatorVOList));
    }


    public ServiceValuatorVO selectById(Long id) {
        log.info("valuator selectById id: {}", id);

        ServiceValuator serviceValuator = this.selectByPrimaryKey(id);
        if (Objects.isNull(serviceValuator)) {
            throw new ServiceException(BizCodeEnum.SERVICE_VALUATOR_NOT_EXIST);
        }

        return AttrCopyUtils.copy(serviceValuator, new ServiceValuatorVO());
    }


    public int save(SaveServiceValuatorDTO dto, Long userId) {
        log.info("valuator save dto: {}, userId: {}", dto, userId);

        // checkAuth(userId);

        Condition c = new Condition(ServiceWorker.class);
        c.createCriteria().andEqualTo("phone", dto.getPhone());
        List<ServiceValuator> serviceValuatorList = this.selectByCondition(c);
        if (!CollectionUtils.isEmpty(serviceValuatorList)) {
            throw new ServiceException(BizCodeEnum.SERVICE_VALUATOR_HAS_EXIST);
        }

        return this.insertSelective(AttrCopyUtils.copy(dto, new ServiceValuator()));
    }



    public List<ServiceValuatorVO> selectByName(String name) {
        log.info("valuator selectByName name: {}", name);

        List<ServiceValuator> serviceValuatorList = serviceValuatorBizMapper.selectByNameLike(name);
        return handle(serviceValuatorList);
    }


    private List<ServiceValuatorVO> handle(List<ServiceValuator> serviceValuatorList){
        if (CollectionUtils.isEmpty(serviceValuatorList)) {
            return Lists.newArrayList();
        }

        // 估价员id列表
        List<Long> valuatorIds = serviceValuatorList.parallelStream().map(ServiceValuator::getId).collect(Collectors.toList());
        // 已被指派的时间
        List<ServiceValuatorAssign> serviceValuatorAssignList = serviceValuatorAssignMapper.selectByDeletedAndIdListAndAssignTime(valuatorIds);
        Map<Long, List<ServiceValuatorAssign>> serviceValuatorAssignMap =
                serviceValuatorAssignList.parallelStream()
                        .collect(Collectors.groupingBy(ServiceValuatorAssign::getValuatorId));

        return serviceValuatorList.parallelStream().map(e1 -> {
            ServiceValuatorVO serviceValuatorVO = AttrCopyUtils.copy(e1, new ServiceValuatorVO());
            // 估价员已经被指派的时间
            List<ServiceValuatorAssign> assigns = serviceValuatorAssignMap.getOrDefault(e1.getId(), Lists.newArrayList());
            List<Date> assignTimes = assigns.parallelStream().map(ServiceValuatorAssign::getAssignTime).collect(Collectors.toList());

            serviceValuatorVO.setAssignTime(assignTimes);

            return serviceValuatorVO;
        }).collect(Collectors.toList());
    }


}
